@@ -14,7 +14,7 @@
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
 
-VERSION = "1.12.1"
+VERSION = "1.13.3"
 
 
 # Set up gettext. This needs to be before the import statements below
@@ -34,7 +34,7 @@ def setup_gettext():
     import gettext
     import os
 
-    domain = 'computerjanitorapp'
+    domain = 'computerjanitor'
     localedir = os.environ.get('LOCPATH', None)
     t = gettext.translation(domain, localedir=localedir, fallback=True)
     return t.ugettext
