@@ -149,7 +149,7 @@ aren't.""")
         for name in ["dash", "gzip"]:
             if name not in self.apt_cache:
                 raise SourcesListProblem(name)
-            if not self.apt_cache[name].candidate.downloadable:
+            if not any(v.downloadable for v in self.apt_cache[name].versions):
                 raise SourcesListProblem(name)
 
     def whitelist_files(self, dirnames):
