@@ -32,7 +32,7 @@ _ = computerjanitorapp.setup_gettext()
 
 import os
 GLADE = os.environ.get('COMPUTER_JANITOR_GLADE', 
-                       '/usr/share/computer-janitor/ComputerJanitor.glade')
+                       '/usr/share/computer-janitor/ComputerJanitor.ui')
 
 
 STATE_COL = 0
@@ -89,11 +89,21 @@ class Cleaner(threading.Thread):
     def __init__(self, ui):
         threading.Thread.__init__(self)
         self.ui = ui
+        self.gtk = self.ui.gtk
         self.crufts = ui.get_crufts()
         self.plugins = ui.pm.get_plugins()
+        # progress reporting dialog/bar
+        # FIXME: the progress bar is a bit silly, it takes
+        #        items like fix-fstab (0.1s) and deb_package (5min)
+        #        and assigns them the same slice - use a pulsing one 
+        #        instead?
         self.dialog = ui.widgets['cleanup_dialog']
         self.pbar = ui.widgets['cleanup_progressbar']
-
+        # add status bar context
+        self.status = self.ui.widgets['statusbar']
+        self.context_id = self.status.get_context_id('ListUpdater')
+        self.status.push(self.context_id, _('Cleaning up...'))
+        # cancel button
         button = ui.widgets['cleanup_cancel']
         button.connect('clicked', self.cancel)
         self.cancel_event = threading.Event()
@@ -101,10 +111,12 @@ class Cleaner(threading.Thread):
     def cancel(self, *args):
         self.cancel_event.set()
         
+    @ui
     def inc_done(self):
         self.done_work += 1.0
         self.pbar.set_fraction(self.done_work / self.total_work)
 
+    @ui
     def start_reporting(self):
         self.total_work = len(self.crufts) + len(self.plugins)
         self.done_work = 0.0
@@ -112,9 +124,11 @@ class Cleaner(threading.Thread):
 
         self.dialog.show()
     
+    @ui
     def end_reporting(self):
         self.pbar.set_fraction(1.0)
         self.dialog.hide()
+        self.status.pop(self.context_id)
 
     def run(self):
         self.ui.cleaning = True
@@ -123,11 +137,12 @@ class Cleaner(threading.Thread):
         self.start_reporting()
         
         for cruft in self.crufts:
-            if self.cancel_event.is_set():
+            if self.cancel_event.isSet():
                 break
             name = cruft.get_name()
             if self.ui.app.state.is_enabled(name):
                 if self.ui.options.no_act:
+                    time.sleep(0.1)
                     logging.info(_("Pretending to remove cruft: %s") % name)
                 else:
                     logging.info(_("Removing cruft: %s") % name)
@@ -135,7 +150,7 @@ class Cleaner(threading.Thread):
             self.inc_done()
 
         for plugin in self.plugins:
-            if self.cancel_event.is_set():
+            if self.cancel_event.isSet():
                 break
             if self.ui.options.no_act:
                 logging.info(_("Pretending to post-cleanup: %s") % plugin)
@@ -151,9 +166,10 @@ class Cleaner(threading.Thread):
                     break
             self.inc_done()
                 
-        self.ui.show_cruft()
+        # do not run it directly (thread deadlocks), register for running
+        # in the gtk main thread instead
+        self.ui.glib.timeout_add(100, self.ui.show_cruft)
         self.ui.cleaning = False
-        self.ui.set_sensitive()
 
         self.end_reporting()
 
@@ -178,10 +194,14 @@ class GtkUserInterface(computerjanitorapp.UserInterface):
         import gobject
         self.gobject = gobject
 
+        import glib
+        self.glib = glib
+
         self.options = options
         self.app.state.load(options.state_file)
 
         builder = gtk.Builder()
+        builder.set_translation_domain('computerjanitor')
         builder.add_from_file(GLADE)
         self.find_and_bind_widgets(builder)
         
@@ -204,15 +224,18 @@ class GtkUserInterface(computerjanitorapp.UserInterface):
         self.finding = False
         self.cleaning = False
 
+        # set thread switches interval to make it more UI friendly
+        sys.setcheckinterval(0)
         gtk.gdk.threads_init()
         gtk.main()
 
     def find_and_bind_widgets(self, builder):
         """Bind widgets and callbacks."""
+        import gtk
         self.widgets = {}
         for o in builder.get_objects():
-            if isinstance(o, self.gtk.Widget):
-                name = o.get_property('name')
+            if issubclass(type(o), gtk.Buildable):
+                name = gtk.Buildable.get_name(o)
                 self.widgets[name] = o
                 for attr in dir(self):
                     prefix = 'on_%s_' % name
@@ -220,6 +243,7 @@ class GtkUserInterface(computerjanitorapp.UserInterface):
                         signal_name = attr[len(prefix):]
                         method = getattr(self, attr)
                         o.connect(signal_name, method)
+                        
     
     def set_default_window_size(self):
         w = self.widgets['window']
@@ -267,6 +291,7 @@ class GtkUserInterface(computerjanitorapp.UserInterface):
     def create_column(self, widget_name, filterfunc):
         """Add gtk.TreeViewColumn to the desired widget."""
         treeview = self.widgets[widget_name]
+        treeview.set_rules_hint(True)
         
         toggle_cr = self.gtk.CellRendererToggle()
         toggle_cr.connect('toggled', self.toggled, treeview)
@@ -347,6 +372,9 @@ class GtkUserInterface(computerjanitorapp.UserInterface):
             sys.exit(1)
 
     def require_working_apt_cache(self):
+        """ensure that the apt cache is in good state and error/exit 
+           otherwise
+        """
         try:
             self.app.verify_apt_cache()
         except computerjanitor.Exception, e:
@@ -356,9 +384,28 @@ class GtkUserInterface(computerjanitorapp.UserInterface):
             dialog.run()
             sys.exit(1)
 
+    def pulse(self):
+        """pulse callback that shows a progress pulse until finding is False"""
+        progress = self.widgets['progressbar_status']
+        if self.finding or self.cleaning:
+            progress.show()
+            progress.pulse()
+            return True
+        else:
+            progress.hide()
+            return False
+
     def show_cruft(self):
+        """clear the cruft store and update it again via a thread """
         self.store.clear()
-        ListUpdater(self).start()
+        # run as "daemon" thread to ensure that the main app exist
+        # if the user presses "quit" before the ListUpdater thread 
+        # has finished
+        t = ListUpdater(self)
+        t.daemon = True
+        t.start()
+        # run a glib handler to shows a pulse progress
+        self.glib.timeout_add(150, self.pulse)
 
     @ui
     def add_cruft(self, cruft):
@@ -383,7 +430,6 @@ class GtkUserInterface(computerjanitorapp.UserInterface):
             dialog.run()
             dialog.hide()
             
-
     def foreach_set_state(self, treeview, enabled):
         def set_state(model, path, iter, user_data):
             iter2 = model.convert_iter_to_child_iter(iter)
@@ -545,7 +591,7 @@ class GtkUserInterface(computerjanitorapp.UserInterface):
     def on_about_menuitem_activate(self, *args):
         w = self.widgets['about_dialog']
         w.set_name(_('Computer Janitor'))
-        w.set_version(computerjanitor.VERSION)
+        w.set_version(computerjanitorapp.VERSION)
         w.show()
         w.run()
         w.hide()
@@ -553,6 +599,7 @@ class GtkUserInterface(computerjanitorapp.UserInterface):
     def on_do_button_clicked(self, *args):
         if self.really_cleanup():
             Cleaner(self).start()
+            self.glib.timeout_add(150, self.pulse)
 
     def on_show_previously_ignored_toggled(self, menuitem):
         self.show_previously_ignored = menuitem.get_active()
@@ -659,3 +706,9 @@ class GtkUserInterface(computerjanitorapp.UserInterface):
             self.sort_crufts_by_current_order = self.sort_crufts_by_size
         self.sort_crufts_by_current_order()
 
+    def on_borednow_messagedialog_close(self, dialog):
+        dialog.hide()
+
+    def on_borednow_messagedialog_response(self, dialog, response):
+        dialog.hide()
+
