@@ -17,7 +17,8 @@
 import os
 
 import computerjanitor
-_ = computerjanitor.setup_gettext()
+import computerjanitorapp
+_ = computerjanitorapp.setup_gettext()
 
 
 class AutoRemovablePlugin(computerjanitor.Plugin):
